@@ -30,15 +30,25 @@
 # --linuxrt (deprecated) Set various options/configurations for doing KAL kernel module work
 #           when run from an NI Linux RT target.  It is unnecessary to set this flag anymore.
 #
-#  version 15.0.1f0
+#  version 17.5.1f0
 #
 #  (C) Copyright 2003-2015,
 #  National Instruments Corporation.
 #  All Rights reserved.
 #
 
-__SELF__=$(if [ -L "$0" ]; then readlink "$0"; else echo "$0"; fi;)
-__DIR__="$(cd "$(dirname "${__SELF__}")" && pwd)"
+# readlink "$0" can return a relative symlink
+# in that case, we need to tread carefully when retrieving the directory
+# for the target of the symlink
+# __TMPSELF__ is the path of the file (if not a symlink), or the target
+# of a symlink, which may be a relative or absolute path
+__TMPSELF__=$(if [ -L "$0" ]; then readlink "$0"; else echo "$0"; fi;)
+__SELF__=$(basename ${__TMPSELF__})
+# In case a symlink was executed, and the symlink target is a relative
+# path, we change to the directory that the path is relative to
+# the directory containing the symlink), then change to the directory
+# containing the symlink target.
+__DIR__="$(cd "$(dirname "$0")" && cd "$(dirname "${__TMPSELF__}")" && pwd)"
 
 # Configure logging
 LOG_MSG_TAG="nikal"
@@ -48,12 +58,12 @@ if [[ -e /usr/share/nikal/installerUtility.sh ]]; then
    source /usr/share/nikal/installerUtility.sh
 elif [[ -e /usr/local/natinst/nikal/bin/installerUtility.sh ]]; then
    source /usr/local/natinst/nikal/bin/installerUtility.sh
-elif [[ -e ${__DIR__}/../../bin/installerUtility.sh ]]; then
-   source ${__DIR__}/../../bin/installerUtility.sh
-elif [[ -e ${__DIR__}/../bin/installerUtility.sh ]]; then
-   source ${__DIR__}/../bin/installerUtility.sh
-elif [[ -e ${__DIR__}/installerUtility.sh ]]; then
-   source ${__DIR__}/installerUtility.sh
+elif [[ -e "${__DIR__}/../../bin/installerUtility.sh" ]]; then
+   source "${__DIR__}/../../bin/installerUtility.sh"
+elif [[ -e "${__DIR__}/../bin/installerUtility.sh" ]]; then
+   source "${__DIR__}/../bin/installerUtility.sh"
+elif [[ -e "${__DIR__}/installerUtility.sh" ]]; then
+   source "${__DIR__}/installerUtility.sh"
 else
    echo "Error opening the NI-KAL driver utility library." >&2
    exit 1
@@ -189,19 +199,19 @@ function parseArguments() {
       return 0
    fi
 
-   if [ "$clientModuleLocation" = "" ]; then
+   if [ -z "$clientModuleLocation" ]; then
       error "Missing required option: client unversioned module location/name not specified!"
       echoUsage
       return 1
    fi
 
-   if [ "$moduleOperation" = "" ]; then
+   if [ -z "$moduleOperation" ]; then
       error "Missing required option: must specify install, remove or exists check with -r -i or -e"
       echoUsage
       return 1
    fi
 
-   if [ "$nikalDir" = "" ] && [ "$moduleOperation" != "output" ]; then
+   if [ -z "$nikalDir" ] && [ "$moduleOperation" != "output" ]; then
       # output mode can work without NI-KAL installed but it is an internal feature
       error "NI-KAL is not installed, please install it before running $0"
       return 1
@@ -232,7 +242,7 @@ function parseArguments() {
    if [ "$moduleOperation" = "output" ]; then
       ldSourceFile=$clientModulePath/$clientModuleUnversioned
    else
-      ldSourceFile=$kalClientDB/$clientInstallPath/$clientModuleUnversioned
+      ldSourceFile=$nikalClientDB/$clientInstallPath/$clientModuleUnversioned
    fi
 }
 
@@ -241,7 +251,7 @@ function parseArguments() {
 #           nonzero for 32-bit or error while determining kernel bitness
 function isKernel64bit() {
    if ! nikalGetConfiguredKernelDir; then
-      return $?
+      return 1
    fi
 
    if [[ -r "${kernelDir}/.config" ]]; then
@@ -256,7 +266,7 @@ function isKernel64bit() {
 #  clientInstallPath : optional sub directory to install in under moduleBasePath
 #  clientModule : name of the installed module, nipalk.o
 #  clientModuleUnversioned : unversioned name of the module, nipalk-unversioned.o
-#  kalClientDB : base directory of the cache where NI-KAL keeps track of client modules
+#  nikalClientDB : base directory of the cache where NI-KAL keeps track of client modules
 #  clientModulePath : Location of the original client unversioned module
 #                     i.e. /usr/local/natinst/nipal/src/objects/nipalk-unversioned.o
 function registerKernelDriver() {
@@ -273,8 +283,8 @@ function registerKernelDriver() {
    fi
 
    # look for previously registered components of the same name
-   if [ -d "$kalClientDB" ]; then
-      numClientsFound=$(find $kalClientDB -name $clientModuleUnversioned | wc -l | sed 's/ //g')
+   if [ -d "$nikalClientDB" ]; then
+      numClientsFound=$(find $nikalClientDB -name $clientModuleUnversioned | wc -l | sed 's/ //g')
 
       if [ "$numClientsFound" -gt 1 ]; then
          # no good, we shouldn't ever have more then one of the same name installed
@@ -282,8 +292,8 @@ function registerKernelDriver() {
          error "This should not be allowed and must be fixed!"
          return 1
       elif [ "$numClientsFound" = "1" ]; then
-         clientFoundName=$(find $kalClientDB -name $clientModuleUnversioned)
-         clientFoundName=$(echo $clientFoundName|sed s#^${kalClientDB}/##)
+         clientFoundName=$(find $nikalClientDB -name $clientModuleUnversioned)
+         clientFoundName=$(echo $clientFoundName|sed s#^${nikalClientDB}/##)
          clientFoundInstallPath=$(echo $clientFoundName|sed s#/${clientModuleUnversioned}##)
 
          # if clientModule was installed previously without -d
@@ -307,15 +317,15 @@ function registerKernelDriver() {
       fi
    fi
 
-   if [ "$clientInstallPath" = "" ]; then
+   if [ -z "$clientInstallPath" ]; then
       warning "The \"-d\" option was not provided to specify a product installation directory."
       warning "Operation will continue but this should be fixed."
       warning " "
       warning "If you are using palmodmgr you should provide the option -o linux:dir=<productDir> to palmodmgr."
    fi
 
-   mkdir -p "$kalClientDB/$clientInstallPath"
-   ln -sf "$clientModulePath/$clientModuleUnversioned" "$kalClientDB/$clientInstallPath/$clientModuleUnversioned"
+   mkdir -p "$nikalClientDB/$clientInstallPath"
+   ln -sf "$clientModulePath/$clientModuleUnversioned" "$nikalClientDB/$clientInstallPath/$clientModuleUnversioned"
    return 0
 }
 
@@ -324,7 +334,7 @@ function registerKernelDriver() {
 #  clientModuleBase : basename of the installed module, nipalk
 #  clientModule : name of the installed module, nipalk.o
 #  clientModuleUnversioned : unversioned name of the module, nipalk-unversioned.o
-#  kalClientDB : base directory of the cache where NI-KAL keeps track of client modules
+#  nikalClientDB : base directory of the cache where NI-KAL keeps track of client modules
 #  clientModulePath : Location of the original client unversioned module
 #                     i.e. /usr/local/natinst/nipal/src/objects/nipalk-unversioned.o
 #  buildDuringInstall : determines whether the module build steps should be invoked
@@ -348,7 +358,7 @@ function installKernelDriver() {
    fi
 
    if ! nikalGetConfiguredKernelModuleBasePath; then
-      return $?
+      return 1
    fi
 
    mkdir -p $moduleBasePath/$clientInstallPath
@@ -367,6 +377,18 @@ function installKernelDriver() {
       fi
       recursiveUnloadKernelDriver "${clientModuleBase}"
       reloadKernelDrivers "${nikalUnloadedModules}"
+
+      # Trigger udev to rematch unmatched devices to kernel modules
+      # this should cause newly-added drivers for hardware in the system
+      # to get loaded and attach to their devices
+      if requirecommands udevadm; then
+         # Tell udevd to re-read the kernel module indexes
+         udevadm control -R
+         # Make sure udevd has processed all outstanding events
+         udevadm settle
+         # Replay (unattached) device events as if they were just coldplugged
+         udevadm trigger
+      fi
    fi
 
    return 0
@@ -377,7 +399,7 @@ function installKernelDriver() {
 #  clientModule : name of the installed module, nipalk.ko
 #  clientModuleBase : basename of the installed module, nipalk
 #  clientModuleUnversioned : unversioned name of the module, nipalk-unversioned.o
-#  kalClientDB : base directory of the cache where NI-KAL keeps track of client modules
+#  nikalClientDB : base directory of the cache where NI-KAL keeps track of client modules
 # Requires a versioning-capable environment
 function removeKernelDriver() {
    if ! requirecommands find sort xargs rmdir; then
@@ -385,10 +407,6 @@ function removeKernelDriver() {
       return 1
    fi
 
-   if ! nikalGetConfiguredKernelModuleBasePath; then
-      return $?
-   fi
-
    # Clean up all installed, configured kernels
    for moddir in /lib/modules/*/kernel/natinst/"${clientInstallPath}"/; do
       rm -f "${moddir}/${clientModule}"
@@ -396,9 +414,9 @@ function removeKernelDriver() {
    done
 
    # Remove client module registration
-   if [ -d "${kalClientDB}/${clientInstallPath}" ]; then
-      rm -f "${kalClientDB}/${clientInstallPath}/${clientModuleUnversioned}"
-      find "${kalClientDB}/${clientInstallPath}/" -type d -depth -exec rmdir '{}' \; > /dev/null 2>&1
+   if [ -d "${nikalClientDB}/${clientInstallPath}" ]; then
+      rm -f "${nikalClientDB}/${clientInstallPath}/${clientModuleUnversioned}"
+      find "${nikalClientDB}/${clientInstallPath}/" -type d -depth -exec rmdir '{}' \; > /dev/null 2>&1
    fi
 
    return 0
@@ -463,19 +481,17 @@ function globalUpdateKernelDriver() {
       error " NI-KAL update failed."
       error " make of nikal kernel module failed, not installing kernel module."
       error " updateNIDrivers should be called again after fixing the problem."
-      error " Logging failure..."
-      nikalCreateSystemReport
       return 1
    fi
 
-   # If there's no kalClientDB directory, then there are no client modules to build
-   if [[ ! -d "$kalClientDB" ]]; then
+   # If there's no nikalClientDB directory, then there are no client modules to build
+   if [[ ! -d "$nikalClientDB" ]]; then
       info "No client modules registered."
       return 0
    fi
 
-   for clientModuleFile in $($nikalDir/src/client/processmodule.sh --cmd_sortdependency --kaldbdir=$kalClientDB); do
-      clientModule=${clientModuleFile#$kalClientDB\/}
+   for clientModuleFile in $($nikalDir/src/client/processmodule.sh --cmd_sortdependency --kaldbdir=$nikalClientDB); do
+      clientModule=${clientModuleFile#$nikalClientDB\/}
       clientInstallPath=${clientModule%\/*}
       clientModuleUnversioned=$(echo $clientModule|sed s#^${clientInstallPath}/## )
       clientModuleBase=$(echo "$clientModuleUnversioned"|sed 's#-unversioned.o##')
@@ -483,7 +499,7 @@ function globalUpdateKernelDriver() {
       ldSourceFile=$clientModuleFile
 
       if ! nikalGetConfiguredKernelModuleBasePath; then
-         return $?
+         return 1
       fi
 
       mkdir -p $moduleBasePath/$clientInstallPath
@@ -519,7 +535,14 @@ function globalUpdateKernelDriver() {
       nikalUnloadedModules=(${!unloadedModules[@]})
       reloadKernelDrivers "${nikalUnloadedModules}"
 
-      if [ "$promptUser" = "0" ]; then
+      # Trigger udev to rematch unmatched devices to kernel modules
+      # this should cause newly-added drivers for hardware in the system
+      # to get loaded and attach to their devices
+      if requirecommands udevadm; then
+         udevadm trigger
+      fi
+
+      if [ "${promptUser}" = "0" ]; then
          promptForReboot
       else
          info "Rebooting is recommended to ensure that all new and updated"
@@ -537,7 +560,7 @@ function globalUpdateKernelDriver() {
 #  clientInstallPath : optional sub directory to install in under moduleBasePath
 #  clientModule : name of the installed module, nipalk.o
 #  clientModuleUnversioned : unversioned name of the module, nipalk-unversioned.o
-#  kalClientDB : base directory of the cache where NI-KAL keeps track of client modules
+#  nikalClientDB : base directory of the cache where NI-KAL keeps track of client modules
 # Requires a versioning-capable environment
 function existsKernelDriver() {
    if ! requirecommands find sed; then
@@ -548,18 +571,18 @@ function existsKernelDriver() {
    statusFound=0
    statusNotFound=1
 
-   if [ ! -d "$kalClientDB" ]; then
+   if [ ! -d "$nikalClientDB" ]; then
       # cache doesn't exist yet so nothing is installed
       return $statusNotFound
    fi
 
    # -e without -d will search the entire cache
-   if [ "$clientInstallPath" = "" ]; then
-      clientFoundName=$(find $kalClientDB -name $clientModuleUnversioned)
-      if [ "$clientFoundName" = "" ]; then
+   if [ -z "$clientInstallPath" ]; then
+      clientFoundName=$(find $nikalClientDB -name $clientModuleUnversioned)
+      if [ -z "$clientFoundName" ]; then
          return $statusNotFound
       else
-         clientInstallPath=$(echo "$clientFoundName" | sed s#^${kalClientDB}/##)
+         clientInstallPath=$(echo "$clientFoundName" | sed s#^${nikalClientDB}/##)
          clientInstallPath=$(echo "$clientInstallPath" | sed s#/${clientModuleUnversioned}##)
 
          # if clientModule was installed previously without -d
@@ -570,10 +593,10 @@ function existsKernelDriver() {
    fi
 
    if ! nikalGetConfiguredKernelModuleBasePath; then
-      return $?
+      return 1
    fi
 
-   if [ ! -L "$kalClientDB/$clientInstallPath/$clientModuleUnversioned" ] ||
+   if [ ! -L "$nikalClientDB/$clientInstallPath/$clientModuleUnversioned" ] ||
       [ ! -r "$moduleBasePath/$clientInstallPath/$clientModule" ]; then
       # not in the cache or not installed
       return $statusNotFound
@@ -608,13 +631,10 @@ forceRebuild=0
 # at module registration time.
 buildDuringInstall=0
 
-if [ -e /etc/natinst/nikal/nikal.dir ]; then
-   nikalDir=$(cat /etc/natinst/nikal/nikal.dir)
-   kalClientDB=$nikalDir/etc/clientkdb
-else
-   # internal use case, from setup.sh buildkernel
-   nikalDir=""
-   kalClientDB=clientkdb
+# Look up nikalDir and nikalClientDB
+if ! nikalGetDirs; then
+   error "Unable to locate a valid NI-KAL installation"
+   exit 1
 fi
 
 #
