@@ -12,6 +12,7 @@
 
 #include <stdbool.h>
 #include <stdarg.h>
+#include <stddef.h>
 
 /* ---------------------------------------------------------------------- */
 /* KAL macros */
@@ -33,7 +34,9 @@ extern "C" {
 #define nNIKAL100_kPCIIOResourceFlagTypeMem  0x2
 #define nNIKAL100_kPCIDontCare (~0)
 
-#ifdef __arm__
+#define nNIKAL1500_kACPIIDLength                9
+
+#if ((defined(__arm__) || defined(__mips__)))
 #define nNIKAL100_cc
 #else
 #define nNIKAL100_cc __attribute__((regparm(0)))
@@ -82,7 +85,7 @@ extern "C" {
 /* KAL types */
 /* ---------------------------------------------------------------------- */
 
-#if ((defined(__GNUG__) || defined(__GNUC__)) && (defined(__i386__) || defined(__arm__)))
+#if ((defined(__GNUG__) || defined(__GNUC__)) && (defined(__i386__) || defined(__arm__) || defined(__mips__)))
    typedef char               nNIKAL100_tText;
    typedef unsigned char      nNIKAL100_tU8;
    typedef char               nNIKAL100_tI8;
@@ -183,6 +186,21 @@ typedef struct
    nNIKAL100_tPCIIOResource ioResource[nNIKAL100_kPCIBARMax];
 } nNIKAL100_tPCIDeviceInfo;
 
+typedef struct
+{
+   nNIKAL100_tU8 id[nNIKAL1500_kACPIIDLength];
+} nNIKAL1500_tACPIDeviceID;
+
+/**
+ * Clients of NI-KAL should not instantiate nNIKAL1500_tACPIDeviceInfo.
+ * One will be provided to the client, when appropriate, via the
+ * nNIKAL100_tDeviceInfo struct.  See the comment above nNIKAL100_tDeviceInfo.
+ **/
+typedef struct
+{
+   nNIKAL1500_tACPIDeviceID deviceID;
+} nNIKAL1500_tACPIDeviceInfo;
+
 typedef struct
 {
    nNIKAL100_tU16 vendor;
@@ -429,7 +447,8 @@ typedef enum
    nNIKAL100_kBusTypePCI,
    nNIKAL100_kBusTypeUSB,
    nNIKAL100_kBusType1394,
-   nNIKAL200_kBusTypeSimulated
+   nNIKAL200_kBusTypeSimulated,
+   nNIKAL1500_kBusTypeACPI
 } nNIKAL100_tBusType;
 
 /**
@@ -449,6 +468,7 @@ typedef struct
    {
       nNIKAL100_tPCIDeviceInfo pci;
       nNIKAL100_tUSBDeviceInfo usb;
+      nNIKAL1500_tACPIDeviceInfo acpi;
    };
 } nNIKAL100_tDeviceInfo;
 
@@ -576,6 +596,20 @@ nNIKAL100_cc nNIKAL100_tI32 nNIKAL100_vsprintf(nNIKAL100_tText* buf, const nNIKA
 nNIKAL100_cc nNIKAL100_tI32 nNIKAL100_vsnPrintf(nNIKAL100_tText* buf, nNIKAL100_tU32 size, const nNIKAL100_tText* fmt, va_list arg);
 nNIKAL100_cc void nNIKAL100_printToDebugger(const nNIKAL100_tText* string);
 
+typedef enum
+{
+   nNIKAL1500_kLogLevelEmergency, /* <== Most severe, syslogd will probably spit this out to all ttys */
+   nNIKAL1500_kLogLevelAlert,
+   nNIKAL1500_kLogLevelCritical,
+   nNIKAL1500_kLogLevelError,
+   nNIKAL1500_kLogLevelWarning,
+   nNIKAL1500_kLogLevelNotice,
+   nNIKAL1500_kLogLevelInfo,
+   nNIKAL1500_kLogLevelDebug, /* <== Least severe */
+} nNIKAL1500_tLogLevel;
+
+nNIKAL100_cc void nNIKAL1500_printLineToDebugger(nNIKAL1500_tLogLevel logLevel, const char *message);
+
 nNIKAL100_cc void *nNIKAL100_malloc(nNIKAL100_tUPtr size);
 nNIKAL100_cc void *nNIKAL100_mallocContiguous(nNIKAL100_tUPtr size);
 nNIKAL100_cc void *nNIKAL100_malloc32BitPhysicalContiguous(nNIKAL100_tUPtr size);
@@ -590,6 +624,8 @@ nNIKAL100_cc int   nNIKAL250_memCmp(const void *buffer1, const void* buffer2, si
 nNIKAL100_cc size_t nNIKAL250_strLen(const char *str);
 nNIKAL100_cc int    nNIKAL250_strCmp(const char *str1, const char *str2);
 nNIKAL100_cc char * nNIKAL250_strnCpy(char *dest, const char *src, size_t count);
+nNIKAL100_cc int    nNIKAL1700_sScanf(const char *str, const char *format, ...);
+nNIKAL100_cc int    nNIKAL1700_vsScanf(const char *str, const char *format, va_list ap);
 
 nNIKAL100_cc nNIKAL100_tU32 nNIKAL100_getPageSize(void);
 nNIKAL100_cc nNIKAL100_tUPtr nNIKAL100_getPhysicalMemorySize(void);
@@ -619,6 +655,7 @@ nNIKAL100_cc nNIKAL100_tU32 nNIKAL100_sleepTimeoutInterruptible(
 nNIKAL100_cc void nNIKAL100_sleepTimeout(nNIKAL100_tU32 timeToSleepInNanoseconds);
 nNIKAL100_cc void nNIKAL250_sleepTimeout(nNIKAL100_tU32 timeToSleepInMicroseconds);
 nNIKAL100_cc void nNIKAL200_yield(void);
+nNIKAL100_cc nNIKAL100_tU64 nNIKAL1_getMonotonicCounter (void);
 nNIKAL100_cc nNIKAL100_tU32 nNIKAL100_getTimerCount (void);
 nNIKAL100_cc nNIKAL100_tU64 nNIKAL110_getTimerCount (void);
 nNIKAL100_cc nNIKAL100_tU32 nNIKAL100_getTimeOfDayInterval (void);
@@ -754,6 +791,7 @@ nNIKAL100_cc nNIKAL100_tU32 nNIKAL100_memoryRead32 ( const nNIKAL100_tUPtr addre
 nNIKAL100_cc nNIKAL100_tI32 nNIKAL240_getPCIDomain(const void * osDevObject);
 
 nNIKAL100_cc nNIKAL100_tStatus nNIKAL230_pciRequestMSI(nNIKAL100_tDeviceInfo *devInfo, nNIKAL100_tU32 numMessages);
+nNIKAL100_cc nNIKAL100_tStatus nNIKAL1700_pciReleaseMSI(nNIKAL100_tDeviceInfo *devInfo);
 
 nNIKAL100_cc nNIKAL100_tStatus nNIKAL100_reserveInterrupt (nNIKAL100_tU32 irq,
    void *context, const nNIKAL100_tText *deviceName);
@@ -784,6 +822,19 @@ nNIKAL100_cc nNIKAL100_tU32 nNIKAL170_getCallStack(nNIKAL100_tU32 callingLevel,
 nNIKAL100_cc void* nNIKAL200_createSimulatedDevice(nNIKAL100_tDriver *driver, const char* devName, void* creationContext);
 nNIKAL100_cc void nNIKAL200_destroySimulatedDevice(nNIKAL100_tDriver *driver, void *kalDevice);
 
+/* This function is provided for use by the processmodule.sh codegen */
+nNIKAL100_cc int nNIKAL1500_helperACPIProbe(void *dev, nNIKAL100_tDriver *driver);
+
+nNIKAL100_cc void *nNIKAL1500_registerACPIDriver(void* inAcpiDriver, void *acpiProbe);
+nNIKAL100_cc void nNIKAL1500_unregisterACPIDriver(void *acpiDriverData);
+nNIKAL100_cc int nNIKAL1500_callACPIMethodGetInteger(const void *osDevObject,
+                                                    const char *method,
+                                                    nNIKAL100_tU64 *value);
+
+nNIKAL100_cc int nNIKAL1500_callACPIMethod(const void *osDevObject,
+                                           const char *method,
+                                           nNIKAL100_tU8 *outbuf, nNIKAL100_tU32 size);
+
 nNIKAL100_cc void* nNIKAL200_getDeviceExtension(void* kalDevice);
 nNIKAL100_cc void* nNIKAL200_getDeviceCreationContext(void* kalDevice);
 
@@ -844,6 +895,7 @@ nNIKAL100_cc nNIKAL100_tStatus nNIKAL220_tUserMemMap_mapPhysicalAddress(nNIKAL22
 
 #define nNIKAL230_kPhysMemFlagDefault       0UL
 #define nNIKAL230_kPhysMemFlagWriteCombined (1UL << 0)
+#define nNIKAL1750_kPhysMemFlagCache        (1UL << 1)
 nNIKAL100_cc nNIKAL100_tStatus nNIKAL230_tUserMemMap_mapPhysicalAddress(nNIKAL220_tUserMemMap *vma,
                                                                  nNIKAL100_tU64 address,
                                                                  size_t size,
@@ -905,8 +957,11 @@ nNIKAL100_cc void nNIKAL100_serialCoreTxWakeup(void *port);
 nNIKAL100_cc void nNIKAL100_serialCoreHandleModemChange(void *port, unsigned int msr);
 nNIKAL100_cc nNIKAL100_tU32 nNIKAL100_serialCoreLockPort(void *port);
 nNIKAL100_cc void nNIKAL100_serialCoreUnlockPort(void *port, nNIKAL100_tU32 flags);
+nNIKAL100_cc nNIKAL100_tBoolean nNIKAL1500_serialCoreIsPortLockedOnWiremodeConfig(void);
 
 nNIKAL100_cc void nNIKAL250_halt(const char *component, const char *file, int line, const char *message);
+nNIKAL100_cc void *nNIKAL1500_registerSysctlInt_debugKALOnly(const char *rootName, const char *sysctlName, int *sysctlVal, unsigned short accessMode);
+nNIKAL100_cc void nNIKAL1500_unregisterSysctl_debugKALOnly(void *sysctlObj);
 
 #ifdef __cplusplus
 }
